@@ -12,7 +12,6 @@ import shutil
 import sys
 import textwrap
 import time
-import yaml
 
 from yaml.error import YAMLError
 
@@ -42,6 +41,7 @@ from ansible.galaxy.role import GalaxyRole
 from ansible.galaxy.token import BasicAuthToken, GalaxyToken, KeycloakToken, NoTokenSentinel
 from ansible.module_utils.ansible_release import __version__ as ansible_version
 from ansible.module_utils.common.collections import is_iterable
+from ansible.module_utils.common.yaml import yaml_dump, yaml_load
 from ansible.module_utils._text import to_bytes, to_native, to_text
 from ansible.module_utils import six
 from ansible.parsing.dataloader import DataLoader
@@ -55,6 +55,17 @@ from ansible.utils.plugin_docs import get_versioned_doclink
 display = Display()
 urlparse = six.moves.urllib.parse.urlparse
 
+SERVER_DEF = [
+    ('url', True),
+    ('username', False),
+    ('password', False),
+    ('token', False),
+    ('auth_url', False),
+    ('v3', False),
+    ('validate_certs', False),
+    ('client_id', False),
+]
+
 
 def with_collection_artifacts_manager(wrapped_method):
     """Inject an artifacts manager if not passed explicitly.
@@ -417,7 +428,7 @@ class GalaxyCLI(CLI):
 
     def add_build_options(self, parser, parents=None):
         build_parser = parser.add_parser('build', parents=parents,
-                                         help='Build an Ansible collection artifact that can be publish to Ansible '
+                                         help='Build an Ansible collection artifact that can be published to Ansible '
                                               'Galaxy.')
         build_parser.set_defaults(func=self.execute_build)
 
@@ -466,11 +477,9 @@ class GalaxyCLI(CLI):
                 ],
                 'required': required,
             }
-        server_def = [('url', True), ('username', False), ('password', False), ('token', False),
-                      ('auth_url', False), ('v3', False)]
 
-        validate_certs = not context.CLIARGS['ignore_certs']
-        galaxy_options = {'validate_certs': validate_certs}
+        validate_certs_fallback = not context.CLIARGS['ignore_certs']
+        galaxy_options = {}
         for optional_key in ['clear_response_cache', 'no_cache']:
             if optional_key in context.CLIARGS:
                 galaxy_options[optional_key] = context.CLIARGS[optional_key]
@@ -482,18 +491,23 @@ class GalaxyCLI(CLI):
         for server_priority, server_key in enumerate(server_list, start=1):
             # Config definitions are looked up dynamically based on the C.GALAXY_SERVER_LIST entry. We look up the
             # section [galaxy_server.<server>] for the values url, username, password, and token.
-            config_dict = dict((k, server_config_def(server_key, k, req)) for k, req in server_def)
-            defs = AnsibleLoader(yaml.safe_dump(config_dict)).get_single_data()
+            config_dict = dict((k, server_config_def(server_key, k, req)) for k, req in SERVER_DEF)
+            defs = AnsibleLoader(yaml_dump(config_dict)).get_single_data()
             C.config.initialize_plugin_configuration_definitions('galaxy_server', server_key, defs)
 
             server_options = C.config.get_plugin_options('galaxy_server', server_key)
             # auth_url is used to create the token, but not directly by GalaxyAPI, so
             # it doesn't need to be passed as kwarg to GalaxyApi
             auth_url = server_options.pop('auth_url', None)
+            client_id = server_options.pop('client_id', None)
             token_val = server_options['token'] or NoTokenSentinel
             username = server_options['username']
             available_api_versions = None
             v3 = server_options.pop('v3', None)
+            validate_certs = server_options['validate_certs']
+            if validate_certs is None:
+                validate_certs = validate_certs_fallback
+            server_options['validate_certs'] = validate_certs
             if v3:
                 # This allows a user to explicitly indicate the server uses the /v3 API
                 # This was added for testing against pulp_ansible and I'm not sure it has
@@ -512,7 +526,8 @@ class GalaxyCLI(CLI):
                     if auth_url:
                         server_options['token'] = KeycloakToken(access_token=token_val,
                                                                 auth_url=auth_url,
-                                                                validate_certs=validate_certs)
+                                                                validate_certs=validate_certs,
+                                                                client_id=client_id)
                     else:
                         # The galaxy v1 / github / django / 'Token'
                         server_options['token'] = GalaxyToken(token=token_val)
@@ -613,7 +628,7 @@ class GalaxyCLI(CLI):
         display.vvv("Reading requirement file at '%s'" % requirements_file)
         with open(b_requirements_file, 'rb') as req_obj:
             try:
-                file_requirements = yaml.safe_load(req_obj)
+                file_requirements = yaml_load(req_obj)
             except YAMLError as err:
                 raise AnsibleError(
                     "Failed to parse the requirements yml at '%s' with the following error:\n%s"
@@ -638,7 +653,7 @@ class GalaxyCLI(CLI):
                 with open(b_include_path, 'rb') as f_include:
                     try:
                         return [GalaxyRole(self.galaxy, self.api, **r) for r in
-                                (RoleRequirement.role_yaml_parse(i) for i in yaml.safe_load(f_include))]
+                                (RoleRequirement.role_yaml_parse(i) for i in yaml_load(f_include))]
                     except Exception as e:
                         raise AnsibleError("Unable to load data from include requirements file: %s %s"
                                            % (to_native(requirements_file), to_native(e)))
@@ -1503,7 +1518,7 @@ class GalaxyCLI(CLI):
         if output_format == 'json':
             display.display(json.dumps(collections_in_paths))
         elif output_format == 'yaml':
-            display.display(yaml.safe_dump(collections_in_paths))
+            display.display(yaml_dump(collections_in_paths))
 
         return 0
 
