@@ -26,6 +26,8 @@ options:
         You can also pass a url or a local path to a rpm file.
         To operate on several packages this can accept a comma separated string of packages or a list of packages."
       - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name>=1.0)
+      - You can also pass an absolute path for a binary which is provided by the package to install.
+        See examples for more information.
     required: true
     aliases:
         - pkg
@@ -107,7 +109,8 @@ options:
     elements: str
   skip_broken:
     description:
-      - Skip packages with broken dependencies(devsolve) and are causing problems.
+      - Skip all unavailable packages or packages with broken dependencies
+        without raising an error. Equivalent to passing the --skip-broken option.
     type: bool
     default: "no"
     version_added: "2.7"
@@ -129,14 +132,14 @@ options:
   security:
     description:
       - If set to C(yes), and C(state=latest) then only installs updates that have been marked security related.
-      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
+      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
     type: bool
     default: "no"
     version_added: "2.7"
   bugfix:
     description:
       - If set to C(yes), and C(state=latest) then only installs updates that have been marked bugfix related.
-      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
+      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
     default: "no"
     type: bool
     version_added: "2.7"
@@ -228,8 +231,31 @@ options:
     type: bool
     default: "no"
     version_added: "2.11"
+  cacheonly:
+    description:
+      - Tells dnf to run entirely from system cache; does not download or update metadata.
+    type: bool
+    default: "no"
+    version_added: "2.12"
+extends_documentation_fragment:
+- action_common_attributes
+- action_common_attributes.flow
+attributes:
+    action:
+        details: In the case of dnf, it has 2 action plugins that use it under the hood, M(ansible.builtin.yum) and M(ansible.builtin.package).
+        support: partial
+    async:
+        support: none
+    bypass_host_loop:
+        support: none
+    check_mode:
+        support: full
+    diff_mode:
+        support: full
+    platform:
+        platforms: rhel
 notes:
-  - When used with a C(loop:) each package will be processed individually, it is much more efficient to pass the list directly to the I(name) option.
+  - When used with a `loop:` each package will be processed individually, it is much more efficient to pass the list directly to the `name` option.
   - Group removal doesn't work if the group was installed with Ansible because
     upstream dnf's API doesn't properly mark groups as installed, therefore upon
     removal the module is unable to detect that the group is installed
@@ -289,6 +315,11 @@ EXAMPLES = '''
     name: /usr/local/src/nginx-release-centos-6-0.el6.ngx.noarch.rpm
     state: present
 
+- name: Install Package based upon the file it provides
+  dnf:
+    name: /usr/bin/cowsay
+    state: present
+
 - name: Install the 'Development tools' package group
   dnf:
     name: '@Development tools'
@@ -327,22 +358,19 @@ import sys
 from ansible.module_utils._text import to_native, to_text
 from ansible.module_utils.urls import fetch_file
 from ansible.module_utils.six import PY2, text_type
-from distutils.version import LooseVersion
+from ansible.module_utils.compat.version import LooseVersion
 
 from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.common.respawn import has_respawned, probe_interpreters_for_module, respawn_module
 from ansible.module_utils.yumdnf import YumDnf, yumdnf_argument_spec
 
-try:
-    import dnf
-    import dnf.cli
-    import dnf.const
-    import dnf.exceptions
-    import dnf.subject
-    import dnf.util
-    HAS_DNF = True
-except ImportError:
-    HAS_DNF = False
+
+# NOTE dnf Python bindings import is postponed, see DnfModule._ensure_dnf(),
+# because we need AnsibleModule object to use get_best_parsable_locale()
+# to set proper locale before importing dnf to be able to scrape
+# the output in some cases (FIXME?).
+dnf = None
 
 
 class DnfModule(YumDnf):
@@ -412,8 +440,13 @@ class DnfModule(YumDnf):
             'release': package.release,
             'version': package.version,
             'repo': package.repoid}
-        result['nevra'] = '{epoch}:{name}-{version}-{release}.{arch}'.format(
-            **result)
+
+        # envra format for alignment with the yum module
+        result['envra'] = '{epoch}:{name}-{version}-{release}.{arch}'.format(**result)
+
+        # keep nevra key for backwards compat as it was previously
+        # defined with a value in envra format
+        result['nevra'] = result['envra']
 
         if package.installtime == 0:
             result['yumstate'] = 'available'
@@ -513,6 +546,21 @@ class DnfModule(YumDnf):
         return rc
 
     def _ensure_dnf(self):
+        locale = get_best_parsable_locale(self.module)
+        os.environ['LC_ALL'] = os.environ['LC_MESSAGES'] = os.environ['LANG'] = locale
+
+        global dnf
+        try:
+            import dnf
+            import dnf.cli
+            import dnf.const
+            import dnf.exceptions
+            import dnf.subject
+            import dnf.util
+            HAS_DNF = True
+        except ImportError:
+            HAS_DNF = False
+
         if HAS_DNF:
             return
 
@@ -612,6 +660,9 @@ class DnfModule(YumDnf):
             if self.download_dir:
                 conf.destdir = self.download_dir
 
+        if self.cacheonly:
+            conf.cacheonly = True
+
         # Default in dnf upstream is true
         conf.clean_requirements_on_remove = self.autoremove
 
@@ -674,15 +725,25 @@ class DnfModule(YumDnf):
                 rc=1
             )
 
-        filters = []
-        if self.bugfix:
-            key = {'advisory_type__eq': 'bugfix'}
-            filters.append(base.sack.query().upgrades().filter(**key))
-        if self.security:
-            key = {'advisory_type__eq': 'security'}
-            filters.append(base.sack.query().upgrades().filter(**key))
-        if filters:
-            base._update_security_filters = filters
+        add_security_filters = getattr(base, "add_security_filters", None)
+        if callable(add_security_filters):
+            filters = {}
+            if self.bugfix:
+                filters.setdefault('types', []).append('bugfix')
+            if self.security:
+                filters.setdefault('types', []).append('security')
+            if filters:
+                add_security_filters('eq', **filters)
+        else:
+            filters = []
+            if self.bugfix:
+                key = {'advisory_type__eq': 'bugfix'}
+                filters.append(base.sack.query().upgrades().filter(**key))
+            if self.security:
+                key = {'advisory_type__eq': 'security'}
+                filters.append(base.sack.query().upgrades().filter(**key))
+            if filters:
+                base._update_security_filters = filters
 
         return base
 
@@ -798,7 +859,7 @@ class DnfModule(YumDnf):
         except dnf.exceptions.DepsolveError as e:
             return {
                 'failed': True,
-                'msg': "Depsolve Error occured for package {0}.".format(pkg_spec),
+                'msg': "Depsolve Error occurred for package {0}.".format(pkg_spec),
                 'failure': " ".join((pkg_spec, to_native(e))),
                 'rc': 1,
                 "results": []
@@ -810,15 +871,19 @@ class DnfModule(YumDnf):
             else:
                 return {
                     'failed': True,
-                    'msg': "Unknown Error occured for package {0}.".format(pkg_spec),
+                    'msg': "Unknown Error occurred for package {0}.".format(pkg_spec),
                     'failure': " ".join((pkg_spec, to_native(e))),
                     'rc': 1,
                     "results": []
                 }
 
     def _whatprovides(self, filepath):
+        self.base.read_all_repos()
         available = self.base.sack.query().available()
-        pkg_spec = available.filter(provides=filepath).run()
+        # Search in file
+        files_filter = available.filter(file=filepath)
+        # And Search in provides
+        pkg_spec = files_filter.union(available.filter(provides=filepath)).run()
 
         if pkg_spec:
             return pkg_spec[0].name
@@ -833,14 +898,13 @@ class DnfModule(YumDnf):
                 filenames.append(name)
             elif name.endswith(".rpm"):
                 filenames.append(name)
-            elif name.startswith("@") or ('/' in name):
+            elif name.startswith('/'):
                 # like "dnf install /usr/bin/vi"
-                if '/' in name:
-                    pkg_spec = self._whatprovides(name)
-                    if pkg_spec:
-                        pkg_specs.append(pkg_spec)
-                        continue
-
+                pkg_spec = self._whatprovides(name)
+                if pkg_spec:
+                    pkg_specs.append(pkg_spec)
+                    continue
+            elif name.startswith("@") or ('/' in name):
                 if not already_loaded_comps:
                     self.base.read_comps()
                     already_loaded_comps = True
@@ -870,7 +934,7 @@ class DnfModule(YumDnf):
                         self.base.package_upgrade(pkg)
                 except Exception as e:
                     self.module.fail_json(
-                        msg="Error occured attempting update_only operation: {0}".format(to_native(e)),
+                        msg="Error occurred attempting update_only operation: {0}".format(to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -890,7 +954,7 @@ class DnfModule(YumDnf):
             except IOError as e:
                 if to_text("Can not load RPM file") in to_text(e):
                     self.module.fail_json(
-                        msg="Error occured attempting remote rpm install of package: {0}. {1}".format(filename, to_native(e)),
+                        msg="Error occurred attempting remote rpm install of package: {0}. {1}".format(filename, to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -906,7 +970,7 @@ class DnfModule(YumDnf):
                         self.base.package_install(pkg)
                 except Exception as e:
                     self.module.fail_json(
-                        msg="Error occured attempting remote rpm operation: {0}".format(to_native(e)),
+                        msg="Error occurred attempting remote rpm operation: {0}".format(to_native(e)),
                         results=[],
                         rc=1,
                     )
@@ -956,7 +1020,7 @@ class DnfModule(YumDnf):
             try:
                 self.base.upgrade_all()
             except dnf.exceptions.DepsolveError as e:
-                failure_response['msg'] = "Depsolve Error occured attempting to upgrade all packages"
+                failure_response['msg'] = "Depsolve Error occurred attempting to upgrade all packages"
                 self.module.fail_json(**failure_response)
         else:
             pkg_specs, group_specs, module_specs, filenames = self._parse_spec_group_file()
@@ -1005,7 +1069,7 @@ class DnfModule(YumDnf):
                         else:
                             response['results'].append("Group {0} installed.".format(group))
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install group: {0}".format(group)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install group: {0}".format(group)
                         self.module.fail_json(**failure_response)
                     except dnf.exceptions.Error as e:
                         # In dnf 2.0 if all the mandatory packages in a group do
@@ -1017,7 +1081,7 @@ class DnfModule(YumDnf):
                     try:
                         self.base.environment_install(environment, dnf.const.GROUP_PACKAGE_TYPES)
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install environment: {0}".format(environment)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install environment: {0}".format(environment)
                         self.module.fail_json(**failure_response)
                     except dnf.exceptions.Error as e:
                         failure_response['failures'].append(" ".join((environment, to_native(e))))
@@ -1085,7 +1149,7 @@ class DnfModule(YumDnf):
                             # If not already installed, try to install.
                             self.base.environment_install(environment, dnf.const.GROUP_PACKAGE_TYPES)
                     except dnf.exceptions.DepsolveError as e:
-                        failure_response['msg'] = "Depsolve Error occured attempting to install environment: {0}".format(environment)
+                        failure_response['msg'] = "Depsolve Error occurred attempting to install environment: {0}".format(environment)
                     except dnf.exceptions.Error as e:
                         failure_response['failures'].append(" ".join((environment, to_native(e))))
 
@@ -1242,14 +1306,18 @@ class DnfModule(YumDnf):
                     # Just return them.
                     self.module.exit_json(**response)
                 else:
-                    self.base.do_transaction()
+                    tid = self.base.do_transaction()
+                    if tid is not None:
+                        transaction = self.base.history.old([tid])[0]
+                        if transaction.return_code:
+                            failure_response['failures'].append(transaction.output())
 
                 if failure_response['failures']:
                     failure_response['msg'] = 'Failed to install some of the specified packages'
-                    self.module.exit_json(**response)
+                    self.module.fail_json(**failure_response)
                 self.module.exit_json(**response)
         except dnf.exceptions.DepsolveError as e:
-            failure_response['msg'] = "Depsolve Error occured: {0}".format(to_native(e))
+            failure_response['msg'] = "Depsolve Error occurred: {0}".format(to_native(e))
             self.module.fail_json(**failure_response)
         except dnf.exceptions.Error as e:
             if to_text("already installed") in to_text(e):
@@ -1257,13 +1325,9 @@ class DnfModule(YumDnf):
                 response['results'].append("Package already installed: {0}".format(to_native(e)))
                 self.module.exit_json(**response)
             else:
-                failure_response['msg'] = "Unknown Error occured: {0}".format(to_native(e))
+                failure_response['msg'] = "Unknown Error occurred: {0}".format(to_native(e))
                 self.module.fail_json(**failure_response)
 
-    @staticmethod
-    def has_dnf():
-        return HAS_DNF
-
     def run(self):
         """The main function."""
 
@@ -1310,7 +1374,7 @@ class DnfModule(YumDnf):
         else:
             # Note: base takes a long time to run so we want to check for failure
             # before running it.
-            if not dnf.util.am_i_root():
+            if not self.download_only and not dnf.util.am_i_root():
                 self.module.fail_json(
                     msg="This command has to be run under the root user.",
                     results=[],
