@@ -19,8 +19,8 @@ short_description: Generic OS package manager
 description:
     - This modules manages packages on a target without specifying a package manager module (like M(ansible.builtin.yum), M(ansible.builtin.apt), ...).
       It is convenient to use in an heterogeneous environment of machines without having to create a specific task for
-      each package manager. C(package) calls behind the module for the package manager used by the operating system
-      discovered by the module M(ansible.builtin.setup).  If C(setup) was not yet run, C(package) will run it.
+      each package manager. `package` calls behind the module for the package manager used by the operating system
+      discovered by the module M(ansible.builtin.setup).  If `setup` was not yet run, `package` will run it.
     - This module acts as a proxy to the underlying package manager module. While all arguments will be passed to the
       underlying module, not all modules support the same arguments. This documentation only covers the minimum intersection
       of module arguments that all packaging modules support.
@@ -39,13 +39,32 @@ options:
     required: true
   use:
     description:
-      - The required package manager module to use (C(yum), C(apt), and so on). The default 'auto' will use existing facts or try to autodetect it.
+      - The required package manager module to use (`yum`, `apt`, and so on). The default 'auto' will use existing facts or try to autodetect it.
       - You should only use this field if the automatic selection is not working for some reason.
     default: auto
 requirements:
     - Whatever is required for the package plugins specific for each system.
+extends_documentation_fragment:
+  -  action_common_attributes
+  -  action_common_attributes.flow
+attributes:
+    action:
+        support: full
+    async:
+        support: full
+    bypass_host_loop:
+        support: none
+    check_mode:
+        details: support depends on the underlying plugin invoked
+        support: N/A
+    diff_mode:
+        details: support depends on the underlying plugin invoked
+        support: N/A
+    platform:
+        details: The support depends on the availability for the specific plugin for each platform and if fact gathering is able to detect it
+        platforms: all
 notes:
-    - While C(package) abstracts package managers to ease dealing with multiple distributions, package name often differs for the same software.
+    - While `package` abstracts package managers to ease dealing with multiple distributions, package name often differs for the same software.
 
 '''
 EXAMPLES = '''
