@@ -28,7 +28,7 @@ import re
 import time
 
 from contextlib import contextmanager
-from distutils.version import LooseVersion
+from ansible.module_utils.compat.version import LooseVersion
 from numbers import Number
 from traceback import format_exc
 
@@ -42,7 +42,15 @@ from jinja2.loaders import FileSystemLoader
 from jinja2.runtime import Context, StrictUndefined
 
 from ansible import constants as C
-from ansible.errors import AnsibleError, AnsibleFilterError, AnsiblePluginRemovedError, AnsibleUndefinedVariable, AnsibleAssertionError
+from ansible.errors import (
+    AnsibleAssertionError,
+    AnsibleError,
+    AnsibleFilterError,
+    AnsibleLookupError,
+    AnsibleOptionsError,
+    AnsiblePluginRemovedError,
+    AnsibleUndefinedVariable,
+)
 from ansible.module_utils.six import iteritems, string_types, text_type
 from ansible.module_utils.six.moves import range
 from ansible.module_utils._text import to_native, to_text, to_bytes
@@ -87,10 +95,11 @@ if C.DEFAULT_JINJA2_NATIVE:
     except ImportError:
         from jinja2 import Environment
         from jinja2.utils import concat as j2_concat
-        display.warning(
-            'jinja2_native requires Jinja 2.10 and above. '
-            'Version detected: %s. Falling back to default.' % j2_version
-        )
+        if C.JINJA2_NATIVE_WARNING:
+            display.warning(
+                'jinja2_native requires Jinja 2.10 and above. '
+                'Version detected: %s. Falling back to default.' % j2_version
+            )
 
 
 JINJA2_BEGIN_TOKENS = frozenset(('variable_begin', 'block_begin', 'comment_begin', 'raw_begin'))
@@ -180,6 +189,25 @@ def _escape_backslashes(data, jinja_env):
     return data
 
 
+def is_possibly_template(data, jinja_env):
+    """Determines if a string looks like a template, by seeing if it
+    contains a jinja2 start delimiter. Does not guarantee that the string
+    is actually a template.
+
+    This is different than ``is_template`` which is more strict.
+    This method may return ``True`` on a string that is not templatable.
+
+    Useful when guarding passing a string for templating, but when
+    you want to allow the templating engine to make the final
+    assessment which may result in ``TemplateSyntaxError``.
+    """
+    if isinstance(data, string_types):
+        for marker in (jinja_env.block_start_string, jinja_env.variable_start_string, jinja_env.comment_start_string):
+            if marker in data:
+                return True
+    return False
+
+
 def is_template(data, jinja_env):
     """This function attempts to quickly detect whether a value is a jinja2
     template. To do so, we look for the first 2 matching jinja2 tokens for
@@ -190,6 +218,11 @@ def is_template(data, jinja_env):
     comment = False
     d2 = jinja_env.preprocess(data)
 
+    # Quick check to see if this is remotely like a template before doing
+    # more expensive investigation.
+    if not is_possibly_template(d2, jinja_env):
+        return False
+
     # This wraps a lot of code, but this is due to lex returning a generator
     # so we may get an exception at any part of the loop
     try:
@@ -430,8 +463,12 @@ class JinjaPluginIntercept(MutableMapping):
             return
 
         for plugin in self._pluginloader.all():
-            method_map = getattr(plugin, self._method_map_name)
-            self._delegatee.update(method_map())
+            try:
+                method_map = getattr(plugin, self._method_map_name)
+                self._delegatee.update(method_map())
+            except Exception as e:
+                display.warning("Skipping %s plugin %s as it seems to be invalid: %r" % (self._dirname, to_text(plugin._original_path), e))
+                continue
 
         if self._pluginloader.class_name == 'FilterModule':
             for plugin_name, plugin in self._delegatee.items():
@@ -445,7 +482,6 @@ class JinjaPluginIntercept(MutableMapping):
     # FUTURE: we can cache FQ filter/test calls for the entire duration of a run, since a given collection's impl's
     # aren't supposed to change during a run
     def __getitem__(self, key):
-        original_key = key
         self._load_ansible_plugins()
 
         try:
@@ -460,61 +496,56 @@ class JinjaPluginIntercept(MutableMapping):
                 if func:
                     return func
 
-            key, leaf_key = get_fqcr_and_name(key)
-            seen = set()
+                # didn't find it in the pre-built Jinja env, assume it's a former builtin and follow the normal routing path
+                leaf_key = key
+                key = 'ansible.builtin.' + key
+            else:
+                leaf_key = key.split('.')[-1]
 
-            while True:
-                if key in seen:
-                    raise TemplateSyntaxError(
-                        'recursive collection redirect found for %r' % original_key,
-                        0
-                    )
-                seen.add(key)
+            acr = AnsibleCollectionRef.try_parse_fqcr(key, self._dirname)
 
-                acr = AnsibleCollectionRef.try_parse_fqcr(key, self._dirname)
+            if not acr:
+                raise KeyError('invalid plugin name: {0}'.format(key))
 
-                if not acr:
-                    raise KeyError('invalid plugin name: {0}'.format(key))
+            ts = _get_collection_metadata(acr.collection)
 
-                ts = _get_collection_metadata(acr.collection)
+            # TODO: implement support for collection-backed redirect (currently only builtin)
+            # TODO: implement cycle detection (unified across collection redir as well)
 
-                # TODO: implement cycle detection (unified across collection redir as well)
+            routing_entry = ts.get('plugin_routing', {}).get(self._dirname, {}).get(leaf_key, {})
 
-                routing_entry = ts.get('plugin_routing', {}).get(self._dirname, {}).get(leaf_key, {})
+            deprecation_entry = routing_entry.get('deprecation')
+            if deprecation_entry:
+                warning_text = deprecation_entry.get('warning_text')
+                removal_date = deprecation_entry.get('removal_date')
+                removal_version = deprecation_entry.get('removal_version')
 
-                deprecation_entry = routing_entry.get('deprecation')
-                if deprecation_entry:
-                    warning_text = deprecation_entry.get('warning_text')
-                    removal_date = deprecation_entry.get('removal_date')
-                    removal_version = deprecation_entry.get('removal_version')
+                if not warning_text:
+                    warning_text = '{0} "{1}" is deprecated'.format(self._dirname, key)
 
-                    if not warning_text:
-                        warning_text = '{0} "{1}" is deprecated'.format(self._dirname, key)
+                display.deprecated(warning_text, version=removal_version, date=removal_date, collection_name=acr.collection)
 
-                    display.deprecated(warning_text, version=removal_version, date=removal_date, collection_name=acr.collection)
+            tombstone_entry = routing_entry.get('tombstone')
 
-                tombstone_entry = routing_entry.get('tombstone')
+            if tombstone_entry:
+                warning_text = tombstone_entry.get('warning_text')
+                removal_date = tombstone_entry.get('removal_date')
+                removal_version = tombstone_entry.get('removal_version')
 
-                if tombstone_entry:
-                    warning_text = tombstone_entry.get('warning_text')
-                    removal_date = tombstone_entry.get('removal_date')
-                    removal_version = tombstone_entry.get('removal_version')
+                if not warning_text:
+                    warning_text = '{0} "{1}" has been removed'.format(self._dirname, key)
 
-                    if not warning_text:
-                        warning_text = '{0} "{1}" has been removed'.format(self._dirname, key)
+                exc_msg = display.get_deprecation_message(warning_text, version=removal_version, date=removal_date,
+                                                          collection_name=acr.collection, removed=True)
 
-                    exc_msg = display.get_deprecation_message(warning_text, version=removal_version, date=removal_date,
-                                                              collection_name=acr.collection, removed=True)
+                raise AnsiblePluginRemovedError(exc_msg)
 
-                    raise AnsiblePluginRemovedError(exc_msg)
-
-                redirect = routing_entry.get('redirect', None)
-                if redirect:
-                    next_key, leaf_key = get_fqcr_and_name(redirect, collection=acr.collection)
-                    display.vvv('redirecting (type: {0}) {1}.{2} to {3}'.format(self._dirname, acr.collection, acr.resource, next_key))
-                    key = next_key
-                else:
-                    break
+            redirect_fqcr = routing_entry.get('redirect', None)
+            if redirect_fqcr:
+                acr = AnsibleCollectionRef.from_fqcr(ref=redirect_fqcr, ref_type=self._dirname)
+                display.vvv('redirecting {0} {1} to {2}.{3}'.format(self._dirname, key, acr.collection, acr.resource))
+                key = redirect_fqcr
+            # TODO: handle recursive forwarding (not necessary for builtin, but definitely for further collection redirs)
 
             func = self._collection_jinja_func_cache.get(key)
 
@@ -544,7 +575,15 @@ class JinjaPluginIntercept(MutableMapping):
 
                 method_map = getattr(plugin_impl, self._method_map_name)
 
-                for func_name, func in iteritems(method_map()):
+                try:
+                    func_items = iteritems(method_map())
+                except Exception as e:
+                    display.warning(
+                        "Skipping %s plugin %s as it seems to be invalid: %r" % (self._dirname, to_text(plugin_impl._original_path), e),
+                    )
+                    continue
+
+                for func_name, func in func_items:
                     fq_name = '.'.join((parent_prefix, func_name))
                     # FIXME: detect/warn on intra-collection function name collisions
                     if self._pluginloader.class_name == 'FilterModule':
@@ -582,17 +621,6 @@ class JinjaPluginIntercept(MutableMapping):
         return len(self._delegatee)
 
 
-def get_fqcr_and_name(resource, collection='ansible.builtin'):
-    if '.' not in resource:
-        name = resource
-        fqcr = collection + '.' + resource
-    else:
-        name = resource.split('.')[-1]
-        fqcr = resource
-
-    return fqcr, name
-
-
 class AnsibleEnvironment(Environment):
     '''
     Our custom environment, which simply allows us to override the class-level
@@ -645,10 +673,6 @@ class Templar:
         self._cached_result = {}
         self._basedir = loader.get_basedir() if loader else './'
 
-        # flags to determine whether certain failures during templating
-        # should result in fatal errors being raised
-        self._fail_on_lookup_errors = True
-        self._fail_on_filter_errors = True
         self._fail_on_undefined_errors = C.DEFAULT_UNDEFINED_VAR_BEHAVIOR
 
         environment_class = AnsibleNativeEnvironment if USE_JINJA2_NATIVE else AnsibleEnvironment
@@ -669,6 +693,7 @@ class Templar:
         self.environment.globals['query'] = self.environment.globals['q'] = self._query_lookup
         self.environment.globals['now'] = self._now_datetime
         self.environment.globals['finalize'] = self._finalize
+        self.environment.globals['undef'] = self._make_undefined
 
         # the current rendering context under which the templar class is working
         self.cur_context = None
@@ -804,107 +829,99 @@ class Templar:
         if fail_on_undefined is None:
             fail_on_undefined = self._fail_on_undefined_errors
 
-        try:
-            if convert_bare:
-                variable = self._convert_bare_variable(variable)
+        if convert_bare:
+            variable = self._convert_bare_variable(variable)
 
-            if isinstance(variable, string_types):
-                result = variable
-
-                if self.is_possibly_template(variable):
-                    # Check to see if the string we are trying to render is just referencing a single
-                    # var.  In this case we don't want to accidentally change the type of the variable
-                    # to a string by using the jinja template renderer. We just want to pass it.
-                    only_one = self.SINGLE_VAR.match(variable)
-                    if only_one:
-                        var_name = only_one.group(1)
-                        if var_name in self._available_variables:
-                            resolved_val = self._available_variables[var_name]
-                            if isinstance(resolved_val, NON_TEMPLATED_TYPES):
-                                return resolved_val
-                            elif resolved_val is None:
-                                return C.DEFAULT_NULL_REPRESENTATION
-
-                    # Using a cache in order to prevent template calls with already templated variables
-                    sha1_hash = None
-                    if cache:
-                        variable_hash = sha1(text_type(variable).encode('utf-8'))
-                        options_hash = sha1(
-                            (
-                                text_type(preserve_trailing_newlines) +
-                                text_type(escape_backslashes) +
-                                text_type(fail_on_undefined) +
-                                text_type(overrides)
-                            ).encode('utf-8')
-                        )
-                        sha1_hash = variable_hash.hexdigest() + options_hash.hexdigest()
-                    if cache and sha1_hash in self._cached_result:
-                        result = self._cached_result[sha1_hash]
-                    else:
-                        result = self.do_template(
-                            variable,
-                            preserve_trailing_newlines=preserve_trailing_newlines,
-                            escape_backslashes=escape_backslashes,
-                            fail_on_undefined=fail_on_undefined,
-                            overrides=overrides,
-                            disable_lookups=disable_lookups,
-                        )
-
-                        if not self.jinja2_native:
-                            unsafe = hasattr(result, '__UNSAFE__')
-                            if convert_data and not self._no_type_regex.match(variable):
-                                # if this looks like a dictionary or list, convert it to such using the safe_eval method
-                                if (result.startswith("{") and not result.startswith(self.environment.variable_start_string)) or \
-                                        result.startswith("[") or result in ("True", "False"):
-                                    eval_results = safe_eval(result, include_exceptions=True)
-                                    if eval_results[1] is None:
-                                        result = eval_results[0]
-                                        if unsafe:
-                                            result = wrap_var(result)
-                                    else:
-                                        # FIXME: if the safe_eval raised an error, should we do something with it?
-                                        pass
-
-                        # we only cache in the case where we have a single variable
-                        # name, to make sure we're not putting things which may otherwise
-                        # be dynamic in the cache (filters, lookups, etc.)
-                        if cache and only_one:
-                            self._cached_result[sha1_hash] = result
-
-                return result
-
-            elif is_sequence(variable):
-                return [self.template(
-                    v,
-                    preserve_trailing_newlines=preserve_trailing_newlines,
-                    fail_on_undefined=fail_on_undefined,
-                    overrides=overrides,
-                    disable_lookups=disable_lookups,
-                ) for v in variable]
-            elif isinstance(variable, Mapping):
-                d = {}
-                # we don't use iteritems() here to avoid problems if the underlying dict
-                # changes sizes due to the templating, which can happen with hostvars
-                for k in variable.keys():
-                    if k not in static_vars:
-                        d[k] = self.template(
-                            variable[k],
-                            preserve_trailing_newlines=preserve_trailing_newlines,
-                            fail_on_undefined=fail_on_undefined,
-                            overrides=overrides,
-                            disable_lookups=disable_lookups,
-                        )
-                    else:
-                        d[k] = variable[k]
-                return d
-            else:
+        if isinstance(variable, string_types):
+            if not self.is_possibly_template(variable):
                 return variable
 
-        except AnsibleFilterError:
-            if self._fail_on_filter_errors:
-                raise
-            else:
-                return variable
+            # Check to see if the string we are trying to render is just referencing a single
+            # var.  In this case we don't want to accidentally change the type of the variable
+            # to a string by using the jinja template renderer. We just want to pass it.
+            only_one = self.SINGLE_VAR.match(variable)
+            if only_one:
+                var_name = only_one.group(1)
+                if var_name in self._available_variables:
+                    resolved_val = self._available_variables[var_name]
+                    if isinstance(resolved_val, NON_TEMPLATED_TYPES):
+                        return resolved_val
+                    elif resolved_val is None:
+                        return C.DEFAULT_NULL_REPRESENTATION
+
+            # Using a cache in order to prevent template calls with already templated variables
+            sha1_hash = None
+            if cache:
+                variable_hash = sha1(text_type(variable).encode('utf-8'))
+                options_hash = sha1(
+                    (
+                        text_type(preserve_trailing_newlines) +
+                        text_type(escape_backslashes) +
+                        text_type(fail_on_undefined) +
+                        text_type(overrides)
+                    ).encode('utf-8')
+                )
+                sha1_hash = variable_hash.hexdigest() + options_hash.hexdigest()
+
+                if sha1_hash in self._cached_result:
+                    return self._cached_result[sha1_hash]
+
+            result = self.do_template(
+                variable,
+                preserve_trailing_newlines=preserve_trailing_newlines,
+                escape_backslashes=escape_backslashes,
+                fail_on_undefined=fail_on_undefined,
+                overrides=overrides,
+                disable_lookups=disable_lookups,
+            )
+
+            if not self.jinja2_native:
+                unsafe = hasattr(result, '__UNSAFE__')
+                if convert_data and not self._no_type_regex.match(variable):
+                    # if this looks like a dictionary or list, convert it to such using the safe_eval method
+                    if (result.startswith("{") and not result.startswith(self.environment.variable_start_string)) or \
+                            result.startswith("[") or result in ("True", "False"):
+                        eval_results = safe_eval(result, include_exceptions=True)
+                        if eval_results[1] is None:
+                            result = eval_results[0]
+                            if unsafe:
+                                result = wrap_var(result)
+                        # FIXME: if the safe_eval raised an error, should we do something with it?
+
+            # we only cache in the case where we have a single variable
+            # name, to make sure we're not putting things which may otherwise
+            # be dynamic in the cache (filters, lookups, etc.)
+            if cache and only_one:
+                self._cached_result[sha1_hash] = result
+
+            return result
+
+        elif is_sequence(variable):
+            return [self.template(
+                v,
+                preserve_trailing_newlines=preserve_trailing_newlines,
+                fail_on_undefined=fail_on_undefined,
+                overrides=overrides,
+                disable_lookups=disable_lookups,
+            ) for v in variable]
+        elif isinstance(variable, Mapping):
+            d = {}
+            # we don't use iteritems() here to avoid problems if the underlying dict
+            # changes sizes due to the templating, which can happen with hostvars
+            for k in variable.keys():
+                if k not in static_vars:
+                    d[k] = self.template(
+                        variable[k],
+                        preserve_trailing_newlines=preserve_trailing_newlines,
+                        fail_on_undefined=fail_on_undefined,
+                        overrides=overrides,
+                        disable_lookups=disable_lookups,
+                    )
+                else:
+                    d[k] = variable[k]
+            return d
+        else:
+            return variable
 
     def is_template(self, data):
         '''lets us know if data has a template'''
@@ -923,23 +940,7 @@ class Templar:
     templatable = is_template
 
     def is_possibly_template(self, data):
-        '''Determines if a string looks like a template, by seeing if it
-        contains a jinja2 start delimiter. Does not guarantee that the string
-        is actually a template.
-
-        This is different than ``is_template`` which is more strict.
-        This method may return ``True`` on a string that is not templatable.
-
-        Useful when guarding passing a string for templating, but when
-        you want to allow the templating engine to make the final
-        assessment which may result in ``TemplateSyntaxError``.
-        '''
-        env = self.environment
-        if isinstance(data, string_types):
-            for marker in (env.block_start_string, env.variable_start_string, env.comment_start_string):
-                if marker in data:
-                    return True
-        return False
+        return is_possibly_template(data, self.environment)
 
     def _convert_bare_variable(self, variable):
         '''
@@ -1014,16 +1015,30 @@ class Templar:
             ran = instance.run(loop_terms, variables=self._available_variables, **kwargs)
         except (AnsibleUndefinedVariable, UndefinedError) as e:
             raise AnsibleUndefinedVariable(e)
+        except AnsibleOptionsError as e:
+            # invalid options given to lookup, just reraise
+            raise e
+        except AnsibleLookupError as e:
+            # lookup handled error but still decided to bail
+            msg = 'Lookup failed but the error is being ignored: %s' % to_native(e)
+            if errors == 'warn':
+                display.warning(msg)
+            elif errors == 'ignore':
+                display.display(msg, log_only=True)
+            else:
+                raise e
+            return [] if wantlist else None
         except Exception as e:
-            if self._fail_on_lookup_errors:
-                msg = u"An unhandled exception occurred while running the lookup plugin '%s'. Error was a %s, original message: %s" % \
-                      (name, type(e), to_text(e))
-                if errors == 'warn':
-                    display.warning(msg)
-                elif errors == 'ignore':
-                    display.display(msg, log_only=True)
-                else:
-                    raise AnsibleError(to_native(msg))
+            # errors not handled by lookup
+            msg = u"An unhandled exception occurred while running the lookup plugin '%s'. Error was a %s, original message: %s" % \
+                  (name, type(e), to_text(e))
+            if errors == 'warn':
+                display.warning(msg)
+            elif errors == 'ignore':
+                display.display(msg, log_only=True)
+            else:
+                display.vvv('exception during Jinja2 execution: {0}'.format(format_exc()))
+                raise AnsibleError(to_native(msg), orig_exc=e)
             return [] if wantlist else None
 
         if ran and allow_unsafe is False:
@@ -1054,6 +1069,13 @@ class Templar:
 
         return ran
 
+    def _make_undefined(self, hint=None):
+        from jinja2.runtime import Undefined
+
+        if hint is None or isinstance(hint, Undefined) or hint == '':
+            hint = "Mandatory variable has not been overridden"
+        return AnsibleUndefined(hint)
+
     def do_template(self, data, preserve_trailing_newlines=True, escape_backslashes=True, fail_on_undefined=None, overrides=None, disable_lookups=False):
         if self.jinja2_native and not isinstance(data, string_types):
             return data
@@ -1107,19 +1129,15 @@ class Templar:
 
             jvars = AnsibleJ2Vars(self, t.globals)
 
-            # In case this is a recursive call to do_template we need to
-            # save/restore cur_context to prevent overriding __UNSAFE__.
-            cached_context = self.cur_context
-
-            self.cur_context = t.new_context(jvars, shared=True)
-            rf = t.root_render_func(self.cur_context)
+            self.cur_context = new_context = t.new_context(jvars, shared=True)
+            rf = t.root_render_func(new_context)
 
             try:
                 if self.jinja2_native:
                     res = ansible_native_concat(rf)
                 else:
                     res = j2_concat(rf)
-                unsafe = getattr(self.cur_context, 'unsafe', False)
+                unsafe = getattr(new_context, 'unsafe', False)
                 if unsafe:
                     res = wrap_var(res)
             except TypeError as te:
@@ -1130,8 +1148,6 @@ class Templar:
                 else:
                     display.debug("failing because of a type error, template data is: %s" % to_text(data))
                     raise AnsibleError("Unexpected templating type error occurred on (%s): %s" % (to_native(data), to_native(te)))
-            finally:
-                self.cur_context = cached_context
 
             if self.jinja2_native and not isinstance(res, string_types):
                 return res
