@@ -69,6 +69,5 @@ ANSIBLE_SSH_TRANSFER_METHOD=piped ./posix.sh "$@"
 # test config defaults override
 ansible-playbook check_ssh_defaults.yml "$@" -i test_connection.inventory
 
-# ensure we handle cp with spaces correctly, otherwise would fail with
-# `"Failed to connect to the host via ssh: command-line line 0: keyword controlpath extra arguments at end of line"`
-ANSIBLE_SSH_CONTROL_PATH='/tmp/ssh cp with spaces' ansible -m ping all -e ansible_connection=ssh -i test_connection.inventory "$@"
+# ensure we can load from ini cfg
+ANSIBLE_CONFIG=./test_ssh_defaults.cfg ansible-playbook verify_config.yml "$@"
