@@ -80,6 +80,8 @@ class TestConnectionBaseClass(unittest.TestCase):
         pc = PlayContext()
         new_stdin = StringIO()
         conn = connection_loader.get('ssh', pc, new_stdin)
+        conn.get_option = MagicMock()
+        conn.get_option.return_value = ""
         conn._build_command('ssh', 'ssh')
 
     def test_plugins_connection_ssh_exec_command(self):
@@ -100,32 +102,34 @@ class TestConnectionBaseClass(unittest.TestCase):
     def test_plugins_connection_ssh__examine_output(self):
         pc = PlayContext()
         new_stdin = StringIO()
-        become_success_token = b'BECOME-SUCCESS-abcdefghijklmnopqrstuvxyz'
 
         conn = connection_loader.get('ssh', pc, new_stdin)
         conn.set_become_plugin(become_loader.get('sudo'))
 
-        conn.check_password_prompt = MagicMock()
-        conn.check_become_success = MagicMock()
-        conn.check_incorrect_password = MagicMock()
-        conn.check_missing_password = MagicMock()
+        conn.become.check_password_prompt = MagicMock()
+        conn.become.check_success = MagicMock()
+        conn.become.check_incorrect_password = MagicMock()
+        conn.become.check_missing_password = MagicMock()
 
         def _check_password_prompt(line):
-            return b'foo' in line
+            if b'foo' in line:
+                return True
+            return False
 
         def _check_become_success(line):
-            return become_success_token in line
+            if b'BECOME-SUCCESS-abcdefghijklmnopqrstuvxyz' in line:
+                return True
+            return False
 
         def _check_incorrect_password(line):
-            return b'incorrect password' in line
+            if b'incorrect password' in line:
+                return True
+            return False
 
         def _check_missing_password(line):
-            return b'bad password' in line
-
-        conn.become.check_password_prompt = MagicMock(side_effect=_check_password_prompt)
-        conn.become.check_become_success = MagicMock(side_effect=_check_become_success)
-        conn.become.check_incorrect_password = MagicMock(side_effect=_check_incorrect_password)
-        conn.become.check_missing_password = MagicMock(side_effect=_check_missing_password)
+            if b'bad password' in line:
+                return True
+            return False
 
         # test examining output for prompt
         conn._flags = dict(
@@ -136,7 +140,13 @@ class TestConnectionBaseClass(unittest.TestCase):
         )
 
         pc.prompt = True
+
+        # override become plugin
         conn.become.prompt = True
+        conn.become.check_password_prompt = MagicMock(side_effect=_check_password_prompt)
+        conn.become.check_success = MagicMock(side_effect=_check_become_success)
+        conn.become.check_incorrect_password = MagicMock(side_effect=_check_incorrect_password)
+        conn.become.check_missing_password = MagicMock(side_effect=_check_missing_password)
 
         def get_option(option):
             if option == 'become_pass':
@@ -162,9 +172,9 @@ class TestConnectionBaseClass(unittest.TestCase):
 
         pc.prompt = False
         conn.become.prompt = False
-        pc.success_key = become_success_token
-        conn.become.success = become_success_token
-        output, unprocessed = conn._examine_output(u'source', u'state', b'line 1\nline 2\n%s\nline 3\n' % become_success_token, False)
+        pc.success_key = u'BECOME-SUCCESS-abcdefghijklmnopqrstuvxyz'
+        conn.become.success = u'BECOME-SUCCESS-abcdefghijklmnopqrstuvxyz'
+        output, unprocessed = conn._examine_output(u'source', u'state', b'line 1\nline 2\nBECOME-SUCCESS-abcdefghijklmnopqrstuvxyz\nline 3\n', False)
         self.assertEqual(output, b'line 1\nline 2\nline 3\n')
         self.assertEqual(unprocessed, b'')
         self.assertFalse(conn._flags['become_prompt'])
@@ -172,23 +182,6 @@ class TestConnectionBaseClass(unittest.TestCase):
         self.assertFalse(conn._flags['become_error'])
         self.assertFalse(conn._flags['become_nopasswd_error'])
 
-        # test we dont detect become success from ssh debug: lines
-        conn._flags = dict(
-            become_prompt=False,
-            become_success=False,
-            become_error=False,
-            become_nopasswd_error=False,
-        )
-
-        pc.prompt = False
-        conn.become.prompt = True
-        pc.success_key = become_success_token
-        conn.become.success = become_success_token
-        output, unprocessed = conn._examine_output(u'source', u'state', b'line 1\nline 2\ndebug1: %s\nline 3\n' % become_success_token, False)
-        self.assertEqual(output, b'line 1\nline 2\ndebug1: %s\nline 3\n' % become_success_token)
-        self.assertEqual(unprocessed, b'')
-        self.assertFalse(conn._flags['become_success'])
-
         # test examining output for become failure
         conn._flags = dict(
             become_prompt=False,
